﻿using System;
using System.Drawing;
using System.Web.UI;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WebEdition;

namespace Serco.Web.Atlas.Public
{
    public partial class WebForm3 : System.Web.UI.Page
    {
        private static RectangleShape englishExtents = new RectangleShape(-645000, 7530000, 210000, 6430000);

        /// <summary>
        /// Handles the Load event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                this.SetupMap(SignsMap);
                this.SetupMap(CctvMap);
            }
        }

        /// <summary>
        /// Set up the specified map control.
        /// </summary>
        /// <param name="map">The map.</param>
        private void SetupMap(Map map)
        {
            // Set viewport, etc on map
            map.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
            map.CurrentExtent = englishExtents;
            map.RestrictedExtent = englishExtents;
            map.MapUnit = GeographyUnit.Meter;
            map.MapTools.MouseCoordinate.Enabled = true;

            // Setup OSM layer on map
            OpenStreetMapOverlay osmOverlay = new OpenStreetMapOverlay("Open Street Map");
            map.CustomOverlays.Add(osmOverlay);

            // Define marker overlay for placing marker pins on map
            InMemoryMarkerOverlay markerOverlay = new InMemoryMarkerOverlay("MarkerOverlay");

            markerOverlay.FeatureSource.Open();
            markerOverlay.Columns.Add(new FeatureSourceColumn("Label"));
            markerOverlay.Columns.Add(new FeatureSourceColumn("Type"));
            markerOverlay.FeatureSource.Close();

            ValueMarkerStyle markerStyle = new ValueMarkerStyle("Type");

            // Small Map Marker, used for single digit marker numbers
            MarkerValueItem marker = new MarkerValueItem("Sign");
            marker.DefaultMarkerStyle.WebImage = new WebImage("/Images/SignMarker.png", 40, 35, 0, -35);
            marker.DefaultMarkerStyle.WebImage.Text = "[#Label#]";
            marker.DefaultMarkerStyle.WebImage.TextOffsetX = 14;
            marker.DefaultMarkerStyle.WebImage.TextOffsetY = 3;
            marker.DefaultMarkerStyle.WebImage.FontColor = GeoColor.StandardColors.Yellow;
            marker.DefaultMarkerStyle.WebImage.FontStyle = new GeoFont("ARIAL", 10, DrawingFontStyles.Bold);
            markerStyle.ValueItems.Add(marker);

            markerOverlay.ZoomLevelSet.ZoomLevel08.CustomMarkerStyle = markerStyle;
            markerOverlay.ZoomLevelSet.ZoomLevel08.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level18;

            // Add marker layer to map
            map.CustomOverlays.Add(markerOverlay);
        }

        /// <summary>
        /// Converts a latitude and longitude to a spherical mercator point.
        /// </summary>
        /// <param name="longitude">The longitude.</param>
        /// <param name="latitude">The latitude.</param>
        /// <returns></returns>
        public static Point LongLatToSphericalMercator(double longitude, double latitude)
        {
            // Spherical Mercator for Google, VE, Yahoo etc
            // epsg:900913 R=6378137, x=longitude, y= R*ln(tan(pi/4 + latitude/2)
            double x = 6378137.0 * Math.PI / 180 * longitude;
            double y = 6378137.0 * Math.Log(Math.Tan(Math.PI / 180 * (45 + latitude / 2.0)));
            return new Point(Convert.ToInt32(x), Convert.ToInt32(y));
        }

        /// <summary>
        /// Handles the Click event of the Button1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="args">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Button1_Click(object sender, EventArgs args)
        {
            AddMarkersToMap(SignsMap);
            ZoomToPoints(SignsMap);

            AddMarkersToMap(CctvMap);
            ZoomToPoints(CctvMap);
        }

        /// <summary>
        /// Adds markers to the specified map.
        /// </summary>
        /// <param name="map">The map.</param>
        private void AddMarkersToMap(Map map)
        {
            // Grab the marker overlay
            InMemoryMarkerOverlay markerOverlay = (InMemoryMarkerOverlay)map.CustomOverlays["MarkerOverlay"];

            Point newPoint1 = LongLatToSphericalMercator(-0.49601668119430542, 51.561748504638672);
            Feature feature1 = new Feature(new PointShape(newPoint1.X, newPoint1.Y));
            feature1.ColumnValues.Add("Label", "1");
            feature1.ColumnValues.Add("Type", "Sign");
            markerOverlay.Features.Add(feature1);

            Point newPoint2 = LongLatToSphericalMercator(-0.5061333179473877, 51.544498443603516);
            Feature feature2 = new Feature(new PointShape(newPoint2.X, newPoint2.Y));
            feature2.ColumnValues.Add("Label", "2");
            feature2.ColumnValues.Add("Type", "Sign");
            markerOverlay.Features.Add(feature2);

            Point newPoint3 = LongLatToSphericalMercator(-0.52176666259765625, 51.559700012207031);
            Feature feature3 = new Feature(new PointShape(newPoint3.X, newPoint3.Y));
            feature3.ColumnValues.Add("Label", "3");
            feature3.ColumnValues.Add("Type", "Sign");
            markerOverlay.Features.Add(feature3);

            // Set the zoom extents
            map.CurrentExtent = new RectangleShape(-296134, 6731273, -292484, 6729076);
        }

        /// <summary>
        /// Finds the bounding rectangle for the points currently present on the specifed map.
        /// </summary>
        /// <param name="map">The map.</param>
        private void ZoomToPoints(Map map)
        {
            double minX = 0;
            double maxX = 0;
            double minY = 0;
            double maxY = 0;
            bool firstItem = true;

            // Check the points in the marker layer
            InMemoryMarkerOverlay markerOverlay = (InMemoryMarkerOverlay)map.CustomOverlays["MarkerOverlay"];

            foreach (Feature feature in markerOverlay.Features)
            {
                PointShape pointShape = (PointShape)feature.GetShape();

                if (firstItem)
                {
                    minX = pointShape.X;
                    maxX = pointShape.X;
                    minY = pointShape.Y;
                    maxY = pointShape.Y;
                    firstItem = false;
                }
                else
                {
                    minX = (pointShape.X < minX) ? pointShape.X : minX;
                    maxX = (pointShape.X > maxX) ? pointShape.X : maxX;
                    maxY = (pointShape.Y > maxY) ? pointShape.Y : maxY;
                    minY = (pointShape.Y < minY) ? pointShape.Y : minY;
                }
            }

            map.CurrentExtent = new RectangleShape(minX, maxY, maxX, minY);
        }
    }
}
